@@ -21,12 +21,19 @@
  * COPYING for details.
  *
  */
-  
+
 #ifndef SDINTER_H_INCLUDED
 #define SDINTER_H_INCLUDED
 
 #include "driver.h"
 #include "logsource.h"
+#include "mainloop-worker.h"
+
+typedef struct AFInterSourceOptions
+{
+  LogSourceOptions super;
+  gint queue_capacity;
+} AFInterSourceOptions;
 
 /*
  * This is the actual source driver, linked into the configuration tree.
@@ -35,7 +42,7 @@ typedef struct _AFInterSourceDriver
 {
   LogSrcDriver super;
   LogSource *source;
-  LogSourceOptions source_options;
+  AFInterSourceOptions source_options;
 } AFInterSourceDriver;
 
 void afinter_postpone_mark(gint mark_freq);
