@@ -47,6 +47,9 @@ typedef struct _CfgParser
   void (*cleanup)(gpointer instance);
 } CfgParser;
 
+gboolean cfg_parser_parse(CfgParser *self, CfgLexer *lexer, gpointer *instance, gpointer arg);
+void cfg_parser_cleanup(CfgParser *self, gpointer instance);
+
 enum
 {
   CFH_SET,
@@ -62,53 +65,23 @@ typedef struct _CfgFlagHandler
   guint32 mask;
 } CfgFlagHandler;
 
-gboolean
-cfg_process_flag(CfgFlagHandler *handlers, gpointer base, const gchar *flag);
+gboolean cfg_process_flag(CfgFlagHandler *handlers, gpointer base, const gchar *flag);
+gboolean cfg_process_yesno(const gchar *yesno);
 
-/* the debug flag for the main parser will be used for all parsers */
-extern int cfg_parser_debug;
-
-static inline gboolean
-cfg_parser_parse(CfgParser *self, CfgLexer *lexer, gpointer *instance, gpointer arg)
-{
-  gboolean success;
-
-  if (cfg_parser_debug)
-    {
-      fprintf(stderr, "\n\nStarting parser %s\n", self->name);
-    }
-  if (self->debug_flag)
-    (*self->debug_flag) = cfg_parser_debug;
-  cfg_lexer_push_context(lexer, self->context, self->keywords, self->name);
-  success = (self->parse(lexer, instance, arg) == 0);
-  cfg_lexer_pop_context(lexer);
-  if (cfg_parser_debug)
-    {
-      fprintf(stderr, "\nStopping parser %s, result: %d\n", self->name, success);
-    }
-  return success;
-}
-
-static inline void
-cfg_parser_cleanup(CfgParser *self, gpointer instance)
-{
-  if (instance && self->cleanup)
-    self->cleanup(instance);
-}
 
 extern CfgParser main_parser;
 
-#define CFG_PARSER_DECLARE_LEXER_BINDING(parser_prefix, root_type)             \
+#define CFG_PARSER_DECLARE_LEXER_BINDING(parser_prefix, PARSER_PREFIX, root_type)             \
     int                                                                        \
-    parser_prefix ## lex(YYSTYPE *yylval, YYLTYPE *yylloc, CfgLexer *lexer);   \
+    parser_prefix ## lex(CFG_STYPE *yylval, CFG_LTYPE *yylloc, CfgLexer *lexer);   \
                                                                                \
     void                                                                       \
-    parser_prefix ## error(YYLTYPE *yylloc, CfgLexer *lexer, root_type instance, gpointer arg, const char *msg);
+    parser_prefix ## error(const CFG_LTYPE *yylloc, CfgLexer *lexer, root_type instance, gpointer arg, const char *msg);
 
 
-#define CFG_PARSER_IMPLEMENT_LEXER_BINDING(parser_prefix, root_type)          \
+#define CFG_PARSER_IMPLEMENT_LEXER_BINDING(parser_prefix, PARSER_PREFIX, root_type)          \
     int                                                                       \
-    parser_prefix ## lex(YYSTYPE *yylval, YYLTYPE *yylloc, CfgLexer *lexer)   \
+    parser_prefix ## lex(CFG_STYPE *yylval, CFG_LTYPE *yylloc, CfgLexer *lexer)   \
     {                                                                         \
       int token;                                                              \
                                                                               \
@@ -117,13 +90,16 @@ extern CfgParser main_parser;
     }                                                                         \
                                                                               \
     void                                                                      \
-    parser_prefix ## error(YYLTYPE *yylloc, CfgLexer *lexer, root_type instance, gpointer arg, const char *msg) \
-    {                                                                                             \
-      report_syntax_error(lexer, yylloc, cfg_lexer_get_context_description(lexer), msg);          \
+    parser_prefix ## error(const CFG_LTYPE *yylloc, CfgLexer *lexer, root_type instance, gpointer arg, const char *msg) \
+    {                                                                 \
+      gboolean in_main_grammar = __builtin_strcmp( # parser_prefix, "main_") == 0;              \
+      report_syntax_error(lexer, yylloc, cfg_lexer_get_context_description(lexer), msg,   \
+                          in_main_grammar);                                                     \
     }
 
-void report_syntax_error(CfgLexer *lexer, YYLTYPE *yylloc, const char *what, const char *msg);
+void report_syntax_error(CfgLexer *lexer, const CFG_LTYPE *yylloc, const char *what, const char *msg,
+                         gboolean in_main_grammar);
 
-CFG_PARSER_DECLARE_LEXER_BINDING(main_, gpointer *)
+CFG_PARSER_DECLARE_LEXER_BINDING(main_, MAIN_, gpointer *)
 
 #endif
