@@ -30,7 +30,6 @@
 
 #define LOGGEN_PLUGIN_INFO "loggen_plugin_info"
 #define LOGGEN_PLUGIN_LIB_PREFIX "libloggen_"
-#define LOGGEN_PLUGIN_LIB_SUFFIX "_plugin.so"
 #define LOGGEN_PLUGIN_NAME_MAXSIZE 100
 
 typedef struct _plugin_option
@@ -46,6 +45,11 @@ typedef struct _plugin_option
   const char *port;
   int  rate;
   int reconnect;
+  gboolean proxied;
+  char *proxy_src_ip;
+  char *proxy_dst_ip;
+  char *proxy_src_port;
+  char *proxy_dst_port;
 } PluginOption;
 
 typedef struct _thread_data
