@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2012 BalaBit IT Ltd, Budapest, Hungary
+ * Copyright (c) 2002-2012 Balabit
  * Copyright (c) 1998-2012 Balázs Scheidler
  *
  * This library is free software; you can redistribute it and/or
@@ -21,15 +21,16 @@
  * COPYING for details.
  *
  */
-  
+
 #ifndef LOGPIPE_H_INCLUDED
 #define LOGPIPE_H_INCLUDED
 
 #include "syslog-ng.h"
-#include "logmsg.h"
+#include "logmsg/logmsg.h"
 #include "cfg.h"
 #include "atomic.h"
 #include "messages.h"
+#include "signal-slot-connector/signal-slot-connector.h"
 
 /* notify code values */
 #define NC_CLOSE       1
@@ -37,8 +38,8 @@
 #define NC_WRITE_ERROR 3
 #define NC_FILE_MOVED  4
 #define NC_FILE_EOF    5
-#define NC_FILE_SKIP   6
-#define NC_REOPEN_REQUIRED 7
+#define NC_REOPEN_REQUIRED 6
+#define NC_FILE_DELETED 7
 
 /* indicates that the LogPipe was initialized */
 #define PIF_INITIALIZED       0x0001
@@ -50,8 +51,10 @@
 #define PIF_BRANCH_FALLBACK   0x0008
 #define PIF_BRANCH_PROPERTIES (PIF_BRANCH_FINAL + PIF_BRANCH_FALLBACK)
 
+#define PIF_DROP_UNMATCHED    0x0010
+
 /* branch starting with this pipe wants hard flow control */
-#define PIF_HARD_FLOW_CONTROL 0x0010
+#define PIF_HARD_FLOW_CONTROL 0x0020
 
 /* this pipe is a source for messages, it is not meant to be used to
  * forward messages, syslog-ng will only use these pipes for the
@@ -59,7 +62,7 @@
  * sending messages to these pipes and these are expected to generate
  * messages "automatically". */
 
-#define PIF_SOURCE            0x0020
+#define PIF_SOURCE            0x0040
 
 /* private flags range, to be used by other LogPipe instances for their own purposes */
 
@@ -167,25 +170,27 @@
  *   The way to override a method by an external object is as follows:
  *
  *     - it should save the current value of the method address (for
- *       example "queue" for the queue method), and the associated
- *       user_data pointer (queue_data in this case)
+ *       example "queue" for the queue method)
  *
  *     - it should change the pointer pointing to the relevant method to
  *       its own code (e.g. change "queue" in LogPipe)
+ *
+ *     - once the hook is invoked, it should take care about calling the
+ *       original function
  **/
 
 struct _LogPathOptions
 {
-   /* an acknowledgement is "passed" to this path, an ACK is still
-    * needed to close the window slot. This was called "flow-control"
-    * and meant both of these things: the user requested
-    * flags(flow-control), _AND_ an acknowledgement was needed. With
-    * the latest change, the one below specifies the user option,
-    * while the "ack is still needed" condition is stored in
-    * ack_needed.
-    */
+  /* an acknowledgement is "passed" to this path, an ACK is still
+   * needed to close the window slot. This was called "flow-control"
+   * and meant both of these things: the user requested
+   * flags(flow-control), _AND_ an acknowledgement was needed. With
+   * the latest change, the one below specifies the user option,
+   * while the "ack is still needed" condition is stored in
+   * ack_needed.
+   */
 
-  gboolean ack_needed:1,
+  gboolean ack_needed;
 
   /* The user has requested flow-control on this processing path,
    * which means that the destination should invoke log_msg_ack()
@@ -199,27 +204,42 @@ struct _LogPathOptions
    * required action.
    */
 
-    flow_control_requested:1;
+  gboolean flow_control_requested;
 
   gboolean *matched;
 };
 
 #define LOG_PATH_OPTIONS_INIT { TRUE, FALSE, NULL }
+#define LOG_PATH_OPTIONS_INIT_NOACK { FALSE, FALSE, NULL }
 
 struct _LogPipe
 {
   GAtomicCounter ref_cnt;
   gint32 flags;
+
+  void (*queue)(LogPipe *self, LogMessage *msg, const LogPathOptions *path_options);
+
   GlobalConfig *cfg;
   LogExprNode *expr_node;
   LogPipe *pipe_next;
+  StatsCounterItem *discarded_messages;
+  const gchar *persist_name;
+  gchar *plugin_name;
+  SignalSlotConnector *signal_slot_connector;
 
-  /* user_data pointer of the "queue" method in case it is overridden
-     by a plugin, see the explanation in the comment on the top. */
-  gpointer queue_data;
-  void (*queue)(LogPipe *self, LogMessage *msg, const LogPathOptions *path_options, gpointer user_data);
+  gboolean (*pre_init)(LogPipe *self);
   gboolean (*init)(LogPipe *self);
   gboolean (*deinit)(LogPipe *self);
+  void (*post_deinit)(LogPipe *self);
+
+  /* this event function is used to perform necessary operation, such as
+   * starting worker thread, and etc. therefore, syslog-ng will terminate if
+   * return value is false.
+   */
+  gboolean (*on_config_inited)(LogPipe *self);
+
+  const gchar *(*generate_persist_name)(const LogPipe *self);
+  GList *(*arcs)(LogPipe *self);
 
   /* clone this pipe when used in multiple locations in the processing
    * pipe-line. If it contains state, it should behave as if it was
@@ -229,20 +249,34 @@ struct _LogPipe
 
   void (*free_fn)(LogPipe *self);
   void (*notify)(LogPipe *self, gint notify_code, gpointer user_data);
+  GList *info;
 };
 
+/*
+   cpu-cache optimization: queue method should be as close as possible to flags.
+
+   Rationale: the pointer pointing to this LogPipe instance is
+   resolved right before calling queue and the colocation of flags and
+   the queue pointer ensures that they are on the same cache line. The
+   queue method is probably the single most often called virtual method
+ */
+G_STATIC_ASSERT(G_STRUCT_OFFSET(LogPipe, queue) - G_STRUCT_OFFSET(LogPipe, flags) <= 4);
+
 extern gboolean (*pipe_single_step_hook)(LogPipe *pipe, LogMessage *msg, const LogPathOptions *path_options);
 
 LogPipe *log_pipe_ref(LogPipe *self);
-void log_pipe_unref(LogPipe *self);
+gboolean log_pipe_unref(LogPipe *self);
 LogPipe *log_pipe_new(GlobalConfig *cfg);
 void log_pipe_init_instance(LogPipe *self, GlobalConfig *cfg);
 void log_pipe_forward_notify(LogPipe *self, gint notify_code, gpointer user_data);
 EVTTAG *log_pipe_location_tag(LogPipe *pipe);
+void log_pipe_attach_expr_node(LogPipe *self, LogExprNode *expr_node);
+void log_pipe_detach_expr_node(LogPipe *self);
 
 static inline GlobalConfig *
 log_pipe_get_config(LogPipe *s)
 {
+  g_assert(s->cfg);
   return s->cfg;
 }
 
@@ -263,6 +297,8 @@ log_pipe_init(LogPipe *s)
 {
   if (!(s->flags & PIF_INITIALIZED))
     {
+      if (s->pre_init && !s->pre_init(s))
+        return FALSE;
       if (!s->init || s->init(s))
         {
           s->flags |= PIF_INITIALIZED;
@@ -281,6 +317,9 @@ log_pipe_deinit(LogPipe *s)
       if (!s->deinit || s->deinit(s))
         {
           s->flags &= ~PIF_INITIALIZED;
+
+          if (s->post_deinit)
+            s->post_deinit(s);
           return TRUE;
         }
       return FALSE;
@@ -288,6 +327,14 @@ log_pipe_deinit(LogPipe *s)
   return TRUE;
 }
 
+static inline gboolean
+log_pipe_on_config_inited(LogPipe *s)
+{
+  if (s->on_config_inited)
+    return s->on_config_inited(s);
+  return TRUE;
+}
+
 static inline void
 log_pipe_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options);
 
@@ -300,54 +347,55 @@ log_pipe_forward_msg(LogPipe *self, LogMessage *msg, const LogPathOptions *path_
     }
   else
     {
-      log_msg_drop(msg, path_options);
+      log_msg_drop(msg, path_options, AT_PROCESSED);
     }
 }
 
 static inline void
 log_pipe_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_options)
 {
+  LogPathOptions local_path_options;
   g_assert((s->flags & PIF_INITIALIZED) != 0);
 
   if (G_UNLIKELY(pipe_single_step_hook))
     {
       if (!pipe_single_step_hook(s, msg, path_options))
         {
-          log_msg_drop(msg, path_options);
+          log_msg_drop(msg, path_options, AT_PROCESSED);
           return;
         }
     }
 
   if (G_UNLIKELY(s->flags & (PIF_HARD_FLOW_CONTROL)))
     {
-      LogPathOptions local_path_options = *path_options;
+      local_path_options = *path_options;
 
       local_path_options.flow_control_requested = 1;
       path_options = &local_path_options;
-      if (G_UNLIKELY(debug_flag))
-        {
-          msg_debug("Requesting flow control",
-                    log_pipe_location_tag(s),
-                    NULL);
-        }
+
+      msg_trace("Requesting flow control", log_pipe_location_tag(s));
     }
 
   if (s->queue)
     {
-      s->queue(s, msg, path_options, s->queue_data);
+      s->queue(s, msg, path_options);
     }
   else
     {
       log_pipe_forward_msg(s, msg, path_options);
     }
+
+  if (path_options->matched && !(*path_options->matched) && (s->flags & PIF_DROP_UNMATCHED))
+    {
+      (*path_options->matched) = TRUE;
+    }
 }
 
 static inline LogPipe *
 log_pipe_clone(LogPipe *self)
 {
-  if (self->clone)
-    return self->clone(self);
-  return NULL;
+  g_assert(NULL != self->clone);
+  return self->clone(self);
 }
 
 static inline void
@@ -363,6 +411,19 @@ log_pipe_append(LogPipe *s, LogPipe *next)
   s->pipe_next = next;
 }
 
+void
+log_pipe_set_persist_name(LogPipe *self, const gchar *persist_name);
+
+const gchar *
+log_pipe_get_persist_name(const LogPipe *self);
+
 void log_pipe_free_method(LogPipe *s);
 
+static inline GList *
+log_pipe_get_arcs(LogPipe *s)
+{
+  return s->arcs(s);
+}
+
+void log_pipe_add_info(LogPipe *self, const gchar *info);
 #endif
