@@ -30,10 +30,12 @@
 
 typedef struct _LogProtoClient LogProtoClient;
 
-#define LOG_PROTO_CLIENT_OPTIONS_SIZE 32
+#define LOG_PROTO_CLIENT_OPTIONS_SIZE 128
 
 typedef struct _LogProtoClientOptions
 {
+  gboolean drop_input;
+  gint timeout;
 } LogProtoClientOptions;
 
 typedef union _LogProtoClientOptionsStorage
@@ -52,6 +54,10 @@ typedef struct
   gpointer user_data;
 } LogProtoClientFlowControlFuncs;
 
+void log_proto_client_options_set_drop_input(LogProtoClientOptions *options, gboolean drop_input);
+void log_proto_client_options_set_timeout(LogProtoClientOptions *options, gint timeout);
+gint log_proto_client_options_get_timeout(LogProtoClientOptions *options);
+
 void log_proto_client_options_defaults(LogProtoClientOptions *options);
 void log_proto_client_options_init(LogProtoClientOptions *options, GlobalConfig *cfg);
 void log_proto_client_options_destroy(LogProtoClientOptions *options);
@@ -62,10 +68,14 @@ struct _LogProtoClient
   const LogProtoClientOptions *options;
   LogTransport *transport;
   /* FIXME: rename to something else */
-  gboolean (*prepare)(LogProtoClient *s, gint *fd, GIOCondition *cond);
-  LogProtoStatus (*post)(LogProtoClient *s, guchar *msg, gsize msg_len, gboolean *consumed);
+  gboolean (*prepare)(LogProtoClient *s, gint *fd, GIOCondition *cond, gint *timeout);
+  LogProtoStatus (*post)(LogProtoClient *s, LogMessage *logmsg, guchar *msg, gsize msg_len, gboolean *consumed);
+  LogProtoStatus (*process_in)(LogProtoClient *s);
   LogProtoStatus (*flush)(LogProtoClient *s);
   gboolean (*validate_options)(LogProtoClient *s);
+  gboolean (*handshake_in_progess)(LogProtoClient *s);
+  LogProtoStatus (*handshake)(LogProtoClient *s);
+  gboolean (*restart_with_state)(LogProtoClient *s, PersistState *state, const gchar *persist_name);
   void (*free_fn)(LogProtoClient *s);
   LogProtoClientFlowControlFuncs flow_control_funcs;
 };
@@ -91,6 +101,12 @@ log_proto_client_msg_rewind(LogProtoClient *self)
     self->flow_control_funcs.rewind_callback(self->flow_control_funcs.user_data);
 }
 
+static inline void
+log_proto_client_set_options(LogProtoClient *self, const LogProtoClientOptions *options)
+{
+  self->options = options;
+}
+
 static inline gboolean
 log_proto_client_validate_options(LogProtoClient *self)
 {
@@ -98,9 +114,29 @@ log_proto_client_validate_options(LogProtoClient *self)
 }
 
 static inline gboolean
-log_proto_client_prepare(LogProtoClient *s, gint *fd, GIOCondition *cond)
+log_proto_client_handshake_in_progress(LogProtoClient *s)
 {
-  return s->prepare(s, fd, cond);
+  if (s->handshake_in_progess)
+    {
+      return s->handshake_in_progess(s);
+    }
+  return FALSE;
+}
+
+static inline LogProtoStatus
+log_proto_client_handshake(LogProtoClient *s)
+{
+  if (s->handshake)
+    {
+      return s->handshake(s);
+    }
+  return LPS_SUCCESS;
+}
+
+static inline gboolean
+log_proto_client_prepare(LogProtoClient *s, gint *fd, GIOCondition *cond, gint *timeout)
+{
+  return s->prepare(s, fd, cond, timeout);
 }
 
 static inline LogProtoStatus
@@ -113,9 +149,26 @@ log_proto_client_flush(LogProtoClient *s)
 }
 
 static inline LogProtoStatus
-log_proto_client_post(LogProtoClient *s, guchar *msg, gsize msg_len, gboolean *consumed)
+log_proto_client_process_in(LogProtoClient *s)
 {
-  return s->post(s, msg, msg_len, consumed);
+  if (s->process_in)
+    return s->process_in(s);
+  else if (s->flush)
+    /*
+     * In some clients, flush is used for input processing, but it should not be.
+     * Fix these clients, than remove the flush call here.
+     *
+     * SSL_ERROR_WANT_READ in the TLS transport is also built upon this.
+     */
+    return s->flush(s);
+  else
+    return LPS_SUCCESS;
+}
+
+static inline LogProtoStatus
+log_proto_client_post(LogProtoClient *s, LogMessage *logmsg, guchar *msg, gsize msg_len, gboolean *consumed)
+{
+  return s->post(s, logmsg, msg, msg_len, consumed);
 }
 
 static inline gint
@@ -131,28 +184,43 @@ log_proto_client_reset_error(LogProtoClient *s)
   s->status = LPS_SUCCESS;
 }
 
+static inline gboolean
+log_proto_client_restart_with_state(LogProtoClient *s, PersistState *state, const gchar *persist_name)
+{
+  if (s->restart_with_state)
+    return s->restart_with_state(s, state, persist_name);
+  return FALSE;
+}
+
 gboolean log_proto_client_validate_options(LogProtoClient *self);
 void log_proto_client_init(LogProtoClient *s, LogTransport *transport, const LogProtoClientOptions *options);
 void log_proto_client_free(LogProtoClient *s);
 void log_proto_client_free_method(LogProtoClient *s);
 
-#define DEFINE_LOG_PROTO_CLIENT(prefix) \
+#define DEFINE_LOG_PROTO_CLIENT(prefix, options...) \
   static gpointer                                                       \
-  prefix ## _client_plugin_construct(Plugin *self,                      \
-                  GlobalConfig *cfg,                                    \
-                  gint plugin_type, const gchar *plugin_name)           \
+  prefix ## _client_plugin_construct(Plugin *self)            \
   {                                                                     \
     static LogProtoClientFactory proto = {                              \
       .construct = prefix ## _client_new,                               \
+      .stateful  = FALSE,                                               \
+      ##options \
     };                                                                  \
     return &proto;                                                      \
   }
 
 #define LOG_PROTO_CLIENT_PLUGIN(prefix, __name) \
-  {							\
-    .type = LL_CONTEXT_CLIENT_PROTO,		        \
-    .name = __name,					\
-    .construct = prefix ## _client_plugin_construct,	\
+  {             \
+    .type = LL_CONTEXT_CLIENT_PROTO,            \
+    .name = __name,         \
+    .construct = prefix ## _client_plugin_construct,  \
+  }
+
+#define LOG_PROTO_CLIENT_PLUGIN_WITH_GRAMMAR(__parser, __name) \
+  {             \
+    .type = LL_CONTEXT_CLIENT_PROTO,            \
+    .name = __name,         \
+    .parser = &__parser,  \
   }
 
 typedef struct _LogProtoClientFactory LogProtoClientFactory;
@@ -160,14 +228,24 @@ typedef struct _LogProtoClientFactory LogProtoClientFactory;
 struct _LogProtoClientFactory
 {
   LogProtoClient *(*construct)(LogTransport *transport, const LogProtoClientOptions *options);
+  gint default_inet_port;
+  gboolean use_multitransport;
+  gboolean stateful;
 };
 
 static inline LogProtoClient *
-log_proto_client_factory_construct(LogProtoClientFactory *self, LogTransport *transport, const LogProtoClientOptions *options)
+log_proto_client_factory_construct(LogProtoClientFactory *self, LogTransport *transport,
+                                   const LogProtoClientOptions *options)
 {
   return self->construct(transport, options);
 }
 
-LogProtoClientFactory *log_proto_client_get_factory(GlobalConfig *cfg, const gchar *name);
+static inline gboolean
+log_proto_client_factory_is_proto_stateful(LogProtoClientFactory *self)
+{
+  return self->stateful;
+}
+
+LogProtoClientFactory *log_proto_client_get_factory(PluginContext *context, const gchar *name);
 
 #endif
