@@ -21,12 +21,14 @@
  * COPYING for details.
  *
  */
-  
+
 #ifndef LOGSOURCE_H_INCLUDED
 #define LOGSOURCE_H_INCLUDED
 
 #include "logpipe.h"
 #include "stats/stats-registry.h"
+#include "window-size-counter.h"
+#include "dynamic-window.h"
 
 typedef struct _LogSourceOptions
 {
@@ -42,6 +44,7 @@ typedef struct _LogSourceOptions
   gint host_override_len;
   LogTagId source_group_tag;
   gboolean read_old_records;
+  gboolean use_syslogng_pid;
   GArray *tags;
   GList *source_queue_callbacks;
   gint stats_level;
@@ -63,32 +66,53 @@ struct _LogSource
   LogPipe super;
   LogSourceOptions *options;
   gboolean threaded;
-  gboolean pos_tracked;
+  gchar *name;
   gchar *stats_id;
   gchar *stats_instance;
-  GAtomicCounter window_size;
-  GAtomicCounter suspended_window_size;
+  WindowSizeCounter window_size;
+  DynamicWindow dynamic_window;
+  gboolean window_initialized;
+  gsize initial_window_size;
+  /* full_window_size = static + dynamic */
+  gsize full_window_size;
+  atomic_gssize window_size_to_be_reclaimed;
+  atomic_gssize pending_reclaimed;
+  StatsCounterItem *stat_window_size;
+  StatsCounterItem *stat_full_window;
   StatsCounterItem *last_message_seen;
   StatsCounterItem *recvd_messages;
+  StatsCluster *stat_window_size_cluster;
+  StatsCluster *stat_full_window_cluster;
+
   guint32 last_ack_count;
   guint32 ack_count;
   glong window_full_sleep_nsec;
   struct timespec last_ack_rate_time;
+  AckTrackerFactory *ack_tracker_factory;
   AckTracker *ack_tracker;
 
   void (*wakeup)(LogSource *s);
+  void (*schedule_dynamic_window_realloc)(LogSource *s);
 };
 
 static inline gboolean
 log_source_free_to_send(LogSource *self)
 {
-  return g_atomic_counter_get(&self->window_size) > 0;
+  return !window_size_counter_suspended(&self->window_size);
 }
 
 static inline gint
 log_source_get_init_window_size(LogSource *self)
 {
-  return self->options->init_window_size;
+  return self->initial_window_size;
+}
+
+static inline void
+log_source_schedule_dynamic_window_realloc(LogSource *s)
+{
+  if (!s || !s->schedule_dynamic_window_realloc)
+    return;
+  s->schedule_dynamic_window_realloc(s);
 }
 
 gboolean log_source_init(LogPipe *s);
@@ -96,7 +120,10 @@ gboolean log_source_deinit(LogPipe *s);
 
 void log_source_post(LogSource *self, LogMessage *msg);
 
-void log_source_set_options(LogSource *self, LogSourceOptions *options, const gchar *stats_id, const gchar *stats_instance, gboolean threaded, gboolean pos_tracked, LogExprNode *expr_node);
+void log_source_set_options(LogSource *self, LogSourceOptions *options, const gchar *stats_id,
+                            const gchar *stats_instance, gboolean threaded, LogExprNode *expr_node);
+void log_source_set_ack_tracker_factory(LogSource *self, AckTrackerFactory *factory);
+void log_source_set_name(LogSource *self, const gchar *name);
 void log_source_mangle_hostname(LogSource *self, LogMessage *msg);
 void log_source_init_instance(LogSource *self, GlobalConfig *cfg);
 void log_source_options_defaults(LogSourceOptions *options);
@@ -106,8 +133,16 @@ void log_source_options_set_tags(LogSourceOptions *options, GList *tags);
 void log_source_free(LogPipe *s);
 void log_source_wakeup(LogSource *self);
 void log_source_flow_control_adjust(LogSource *self, guint32 window_size_increment);
+void log_source_flow_control_adjust_when_suspended(LogSource *self, guint32 window_size_increment);
 void log_source_flow_control_suspend(LogSource *self);
+void log_source_disable_bookmark_saving(LogSource *self);
+void log_source_enable_dynamic_window(LogSource *self, DynamicWindowPool *window_ctr);
+void log_source_dynamic_window_update_statistics(LogSource *self);
+gboolean log_source_is_dynamic_window_enabled(LogSource *self);
 
 void log_source_global_init(void);
 
+/* protected */
+void log_source_dynamic_window_realloc(LogSource *self);
+
 #endif
