@@ -61,6 +61,7 @@ enum
   M_RUNID,
   M_HOSTID,
   M_UNIQID,
+  M__ASTERISK,
 
   /* only touch this section if you want to add three macros, one w/o
    * prefix, and a R_ and S_ prefixed macro that relates one of the
@@ -117,9 +118,11 @@ extern LogMacroDef macros[];
 
 /* low level macro functions */
 guint log_macro_lookup(const gchar *macro, gint len);
-gboolean log_macro_expand(GString *result, gint id, gboolean escape, LogTemplateEvalOptions *options,
-                          const LogMessage *msg);
-gboolean log_macro_expand_simple(GString *result, gint id, const LogMessage *msg);
+gboolean log_macro_expand(gint id, gboolean escape, LogTemplateEvalOptions *options,
+                          const LogMessage *msg,
+                          GString *result, LogMessageValueType *type);
+gboolean log_macro_expand_simple(gint id, const LogMessage *msg,
+                                 GString *result, LogMessageValueType *type);
 
 void log_macros_global_init(void);
 void log_macros_global_deinit(void);
