@@ -29,22 +29,32 @@
 typedef struct _LogTransportAuxData
 {
   GSockAddr *peer_addr;
-  gchar data[1024];
+  GSockAddr *local_addr;
+  gint proto;
+  gchar data[1536];
   gsize end_ptr;
 } LogTransportAuxData;
 
 static inline void
 log_transport_aux_data_init(LogTransportAuxData *self)
 {
-  self->peer_addr = NULL;
-  self->end_ptr = 0;
-  self->data[0] = 0;
+  if (self)
+    {
+      self->peer_addr = NULL;
+      self->local_addr = NULL;
+      self->end_ptr = 0;
+      self->data[0] = 0;
+    }
 }
 
 static inline void
 log_transport_aux_data_destroy(LogTransportAuxData *self)
 {
-  g_sockaddr_unref(self->peer_addr);
+  if (self)
+    {
+      g_sockaddr_unref(self->peer_addr);
+      g_sockaddr_unref(self->local_addr);
+    }
 }
 
 static inline void
@@ -59,19 +69,38 @@ log_transport_aux_data_copy(LogTransportAuxData *dst, LogTransportAuxData *src)
 {
   gsize data_to_copy = sizeof(*src) - sizeof(src->data) + src->end_ptr;
 
-  memcpy(dst, src, data_to_copy);
-  g_sockaddr_ref(dst->peer_addr);
+  if (dst)
+    {
+      memcpy(dst, src, data_to_copy);
+      g_sockaddr_ref(dst->peer_addr);
+      g_sockaddr_ref(dst->local_addr);
+    }
 }
 
 static inline void
 log_transport_aux_data_set_peer_addr_ref(LogTransportAuxData *self, GSockAddr *peer_addr)
 {
-  if (self->peer_addr)
-    g_sockaddr_unref(self->peer_addr);
-  self->peer_addr = peer_addr;
+  if (self)
+    {
+      if (self->peer_addr)
+        g_sockaddr_unref(self->peer_addr);
+      self->peer_addr = peer_addr;
+    }
+}
+
+static inline void
+log_transport_aux_data_set_local_addr_ref(LogTransportAuxData *self, GSockAddr *local_addr)
+{
+  if (self)
+    {
+      if (self->local_addr)
+        g_sockaddr_unref(self->local_addr);
+      self->local_addr = local_addr;
+    }
 }
 
 void log_transport_aux_data_add_nv_pair(LogTransportAuxData *self, const gchar *name, const gchar *value);
-void log_transport_aux_data_foreach(LogTransportAuxData *self, void (*func)(const gchar *, const gchar *, gsize, gpointer), gpointer user_data);
+void log_transport_aux_data_foreach(LogTransportAuxData *self, void (*func)(const gchar *, const gchar *, gsize,
+                                    gpointer), gpointer user_data);
 
 #endif
